#include <stdio.h>

#include "node/printers.h"
#include "node/node.h"

void print_node(Node* n)
{
    _fprint_node(n, stdout);
}

void fprint_node(Node* n, int fd)
{
    FILE* f = fdopen(fd, "w");
    _fprint_node(n, f);
    fclose(f);
}

void _fprint_node(Node* n, FILE* f)
{
    switch (n->opcode) {
        case OP_ADD:    add_p(n, f); break;
        case OP_SUB:    sub_p(n, f); break;
        case OP_MUL:    mul_p(n, f); break;
        case OP_DIV:    div_p(n, f); break;
        case OP_MIN:    min_p(n, f); break;
        case OP_MAX:    max_p(n, f); break;
        case OP_POW:    pow_p(n, f); break;
        
        case OP_SQUARE: square_p(n, f); break;
        case OP_SQRT:   sqrt_p(n, f); break;
        case OP_SIN:    sin_p(n, f); break;
        case OP_COS:    cos_p(n, f); break;
        case OP_TAN:    tan_p(n, f); break;
        case OP_NEG:    neg_p(n, f); break;
        case OP_BUF:    buf_p(n, f); break;
        
        case OP_CONST:  constant_p(n, f); break;
        case OP_X:      X_p(n, f); break;
        case OP_Y:      Y_p(n, f); break;
        case OP_Z:      Z_p(n, f); break;
        default:
            fprintf(f, "Unknown opcode!\n");
    }
}



void add_p(Node* n, FILE* f)
{
    fprintf(f, "(");
    _fprint_node(n->lhs, f);
    fprintf(f, "+");
    _fprint_node(n->rhs, f);
    fprintf(f, ")");
}

void sub_p(Node* n, FILE* f)
{
    fprintf(f, "(");
    _fprint_node(n->lhs, f);
    fprintf(f, "-");
    _fprint_node(n->rhs, f);
    fprintf(f, ")");
}

void mul_p(Node* n, FILE* f)
{
    fprintf(f, "(");
    _fprint_node(n->lhs, f);
    fprintf(f, "*");
    _fprint_node(n->rhs, f);
    fprintf(f, ")");
}

void div_p(Node* n, FILE* f)
{
    fprintf(f, "(");
    _fprint_node(n->lhs, f);
    fprintf(f, "/");
    _fprint_node(n->rhs, f);
    fprintf(f, ")");
}

void min_p(Node* n, FILE* f)
{
    fprintf(f, "min(");
    _fprint_node(n->lhs, f);
    fprintf(f, ", ");
    _fprint_node(n->rhs, f);
    fprintf(f, ")");
}

void max_p(Node* n, FILE* f)
{
    fprintf(f, "max(");
    _fprint_node(n->lhs, f);
    fprintf(f, ", ");
    _fprint_node(n->rhs, f);
    fprintf(f, ")");
}

void pow_p(Node* n, FILE* f)
{
    fprintf(f, "pow(");
    _fprint_node(n->lhs, f);
    fprintf(f, ", ");
    _fprint_node(n->rhs, f);
    fprintf(f, ")");
}

////////////////////////////////////////////////////////////////////////////////

void square_p(Node* n, FILE* f)
{
    fprintf(f, "pow(");
    _fprint_node(n->lhs, f);
    fprintf(f, ", 2)");
}

void sqrt_p(Node* n, FILE* f)
{
    fprintf(f, "sqrt(");
    _fprint_node(n->lhs, f);
    fprintf(f, ")");
}

void sin_p(Node* n, FILE* f)
{
    fprintf(f, "sin(");
    _fprint_node(n->lhs, f);
    fprintf(f, ")");
}

void cos_p(Node* n, FILE* f)
{
    fprintf(f, "cos(");
    _fprint_node(n->lhs, f);
    fprintf(f, ")");
}

void tan_p(Node* n, FILE* f)
{
    fprintf(f, "tan(");
    _fprint_node(n->lhs, f);
    fprintf(f, ")");
}

void neg_p(Node* n, FILE* f)
{
    fprintf(f, "-");
    _fprint_node(n->lhs, f);
}

void buf_p(Node* n, FILE* f)
{
    _fprint_node(n->lhs, f);
}

////////////////////////////////////////////////////////////////////////////////

void constant_p(Node* n, FILE* f)
{
    fprintf(f, "%g", n->results.f);
}

void X_p(Node* n, FILE* f)
{
    fprintf(f, "X");
}

void Y_p(Node* n, FILE* f)
{
    fprintf(f, "Y");
}

void Z_p(Node* n, FILE* f)
{
    fprintf(f, "Z");
}