import threading

def monitor(interrupt, halt):
    interrupt.wait()
    halt.value = 1
    
def multithread(target, args, interrupt=None, halt=None):
    ''' Spreads a process across multiple threads. '''
    
    if (halt is None) ^ (interrupt is None):
        raise ValueError('multithread must be invoked with both halt and interrupt (or neither)')
    
    threads = [threading.Thread(target=target, args=a) for a in args]
    
    if interrupt:
        m = threading.Thread(target=monitor, args=(interrupt, halt))
        m.daemon = True
        m.start()
    
    for t in threads:   t.daemon = True
    for t in threads:   t.start()
    for t in threads:   t.join()
    
    if interrupt:
        interrupt.set()
        m.join()
        interrupt.clear()    